#!/usr/bin/env ruby
# namazu.rb - Namazu search client
# $Id: namazu.rb,v 1.2 2000/07/05 15:53:25 ryu Exp $
#
# namazu.rb is copyrighted free software by Ryunosuke Ohshima.
# You can use/redistribute/modify it under the terms of Namazu or Ruby.

require 'rbnamazu'

if ENV['GATEWAY_INTERFACE']

  require 'cgi'
  cgi = CGI.new()
  params = cgi.params()
  params.each() do |key, value|
    params[key] = value.join(' ')
  end
  nmzoptions = {
    'max' => 20,
    'html' => true,
    'references' => true,
    'page' => true,
    'form' => true,
    'replace' => true,
    'decode-uri' => true,
  }
  nmzoptions.update(cgi.params())

  $0 = (ENV['PATH_TRANSLATED'] or ENV['SCRIPT_FILENAME'] or $0)
  nmzoptions['config'] = File::dirname($0) + File::Separator + '.namazurc'

  require 'nkf'
  querystring = NKF::nkf('-emXZ1', nmzoptions['query'])

  nmz = Namazu.new(Array.new(), nmzoptions, Namazu::CGIFormatter)
  puts nmz.result(querystring)

elsif __FILE__ == $0

  require 'optparse'
  
  nmzoptions = {}
  ARGV.options do |opts|
    opts.banner = 'Usage: ' + File::basename($0) + ' [options] <query> [index]...' + "\n"

    opts.on('-n', '--max=NUM', Integer,
	    'set number of documents shown to NUM.') {|nmzoptions['max']|}
    opts.on('-w', '--whence=NUM', Integer,
	    'set first number of documents shown to NUM.') {|nmzoptions['whence']|}
    opts.on('-l', '--list',
	    'print results by listing format.') {|nmzoptions['list']|}
    opts.on('-s', '--short',
	    'print results by short format.'){|nmzoptions['short']|}
    opts.on('--result=EXT', String,
	    'set NMZ.result.EXT for printing results.') {|nmzoptions['result']|}
    opts.on('--late',
	    'sort documents in late order.') {|nmzoptions['late']|}
    opts.on('--early',
	    'sort documents in early order.') {|nmzoptions['early']|}
    opts.on('--sort=METHOD', String,
	    'set a sort METHOD (score, date, field:name)') {|nmzoptions['sort']|}
    opts.on('--ascending',
	    'sort in ascending order (default: descending)') {|nmzoptions['ascending']|}
    #    opts.on('-a','--all',
    #	    'print all results.') {|nmzoptions['all']|}
    opts.on('-c','--count',
	    'print only number of hits.') {|nmzoptions['count']|}
    opts.on('-h','--html',
	    'print in HTML format.') {|nmzoptions['html']|}
    nmzoptions['references'] = true
    opts.on('-r','--no-references',
	    'do not display reference hit counts.') {|nmzoptions['references']|}
    opts.on('-H','--page',
	    'print further result links. (nearly meaningless)') {|nmzoptions['page']|}
    opts.on('-F','--form',
	    'force to print <form> ... </form> region.') {|nmzoptions['form']|}
    nmzoptions['replace'] = true
    opts.on('-R','--no-replace',
	    'do not replace URI string.') {|nmzoptions['replace']|}
    nmzoptions['decode-uri'] = true
    opts.on('-U','--no-decode-uri',
	    'do not decode URI when printing in a plain format.') {|nmzoptions['decode-uri']|}
    #    opts.on('-o','--output=FILE', String,
    #	    'set the output file name to FILE.') {|nmzoptions['output']|}
    opts.on('-f','--config=FILE', String,
	    'set the config file name to FILE.') {|nmzoptions['config']|}
    opts.on('-C','--show-config',
	    'print current configuration.') {|nmzoptions['show-config']|}
    #    opts.on('-q','--quiet',
    #	    'do not display extra messages except search results.') {|nmzoptions['quiet']|}
    #    opts.on('-d','--debug',
    #	    'be debug mode.') {|nmzoptions['debug']|}
    opts.on('-v','--version',
	    'show the version of namazu and exit.') {puts File::basename($0) + '-0.3'; exit}
    opts.on('--help',
	    'show this help and exit') {puts opts; exit}

    opts.parse!
  end

  querystring, nmzdirname = ARGV
  if querystring
    nmzdirnames = Array.new()
    nmzdirnames.push(nmzdirname) if nmzdirname
    nmz = Namazu.new(nmzdirnames, nmzoptions)
    puts nmz.result(querystring)
  else
    puts 'Usage: ' + File::basename($0) + ' [options] <query> [index]...'
    puts 'Try `' + File::basename($0) + ' --help\' for more information.'
  end
end
