# nmzdoc.rb - libraries for making documents of Namazu client
# $Id: nmzdoc.rb,v 1.2 2000/07/05 16:00:21 ryu Exp $
#
# nmzdoc.rb is copyrighted free software by Ryunosuke Ohshima.
# You can use/redistribute/modify it under the terms of Namazu or Ruby.

class Namazu
  class Config
    def initialize(nmzconfname = nil)
      @config = Hash.new()
      unless nmzconfname
	tmpnmzconfname = Array.new()
	tmpnmzconfname.push(ENV['HOME'] + File::Separator + '.namazurc') if ENV['HOME']
	tmpnmzconfname.push(ENV['NAMAZURC']) if ENV['NAMAZURC']
	tmpnmzconfname.push(['', 'usr', 'local', 'etc', 'namazurc', 'namazurc'].join(File::Separator))
	for nmzconfname in tmpnmzconfname
	  break if FileTest::file?(nmzconfname)
	end
      end
      if FileTest::readable?(nmzconfname)
	open(nmzconfname) do |conffile|
	  conffile.each_line() do |line|
	    cindex = line.index(/\s*\#/)
	    if cindex
	      line[cindex .. line.length() - 1] = ''
	    end
	    if /REPLACE\s+(\S+)\s+(\S+)/i =~ line
	      @config['replace'] = Array.new() unless @config['replace']
	      @config['replace'].push([$1, $2])
	    elsif /(\w+)\s+(.+)/ =~ line
	      @config[$1.downcase()] = $2
	    end
	  end
	end
      end
    end
    attr_reader(:config)

    public
    def show_config()
      @config.each() do |name, value|
	if value.kind_of?(Array)
	  value.each() do |val|
	    if val.kind_of?(Array)
	      puts(name.capitalize() + "\t: " + val.join("\t"))
	    else
	      puts(name.capitalize() + "\t: " + val)
	    end
	  end
	else
	  puts(name.capitalize() + "\t: " + value)
	end
      end
    end

    public
    def [](arg)
      @config[arg]
    end
  end # Config

  class Document < Hash
  end # Document

  class Summary < Array
    attr_accessor(:querystring, :hitnum, :hitref, :whence, :max)

    public
    def get_whence(whence = nil)
      newwhence = 0
      if whence
	newwhence = [whence, newwhence].max()
      end
      newwhence
    end

    public
    def get_max(max = nil)
      newmax = [length() - whence(), 0].max()
      if max
	newmax = [max, newmax].min()
      end
      newmax
    end

    public
    def [](*arg)
      if arg.length() == 2
	whence = get_whence(arg[0])
	max = get_max(arg[1])
	if whence == 0 and max == length()
	  self
	else
	  summary = Summary.new()
	  summary[whence, max] = super(whence, max)
	  summary.querystring = querystring()
	  summary.hitnum = hitnum()
	  summary.hitref = hitref()
	  summary.whence = whence
	  summary.max = max
	  summary
	end
      else
	super
      end
    end
  end # Summary

  class Summarizer
    def initialize(nmzindices, nmzoptions = Hash.new())
      @nmzindices = nmzindices
      @nmzoptions = nmzoptions
      @summary = nil
    end
    attr_reader(:summary)

    public
    def get_whence(whence = nil)
      if whence
	@summary.get_whence(whence)
      elsif @nmzoptions['whence']
	@summary.get_whence(@nmzoptions['whence'].to_i())
      else
	@summary.get_whence()
      end
    end

    public
    def get_max(max = nil)
      if max
	@summary.get_max(max)
      elsif @nmzoptions['max']
	@summary.get_max(@nmzoptions['max'].to_i())
      else
	@summary.get_max()
      end
    end

    public
    def summarize(querystring)
      @summary = nil
      if querystring
	nmzresults = @nmzindices.collect() do |nmzindex|
	  nmzindex.search(querystring)
	end
	if nmzresults.compact().length() > 0
	  @summary = Summary.new()
	  @summary.querystring = querystring
	  @summary.hitref = ''
	  for i in 0 .. nmzresults.length() - 1
	    nmzresult = nmzresults[i]
	    next unless nmzresult
	    nmzindex = @nmzindices[i]
	    for docid in nmzresult.docids()
	      nmzdoc = Document.new()
	      nmzdoc['docid'] = docid
	      nmzdoc['score'] = nmzresult.score(docid)
	      nmzdoc['time'] = nmzresult.time(docid)
	      nmzdoc.update(nmzindex.nmzfieldall().gets(docid))
	      @summary.push(nmzdoc)
	    end
	  end
	  @summary.hitref = nmzresults.collect() do |nmzresult|
	    nmzresult ? nmzresult.to_s() : ' (no result) '
	  end .join(' ::: ')
	  @summary.hitnum = @summary.length()
	  @summary.whence = get_whence()
	  @summary.max = get_max()
	end
      end
      @summary
    end

    public
    def sort!(nmzoptions = nil)
      if nmzoptions
	@nmzoptions = nmzoptions
      end
      @summary.sort!() { |a, b| b['score'] <=> a['score'] }
      @summary.sort!() { |a, b| b['time'] <=> a['time'] } if @nmzoptions['late']
      @summary.sort!() { |a, b| a['time'] <=> b['time'] } if @nmzoptions['early']
      if @nmzoptions['sort']
	if /date/i =~ @nmzoptions['sort']
	  @summary.sort!() { |a, b| b['time'] <=> a['time'] }
	  @summary.reverse!() if /early/i =~ @nmzoptions['sort']
	elsif /field:([^:]*)(:.*)?/i =~ @nmzoptions['sort']
	  field = $1
	  if field == 'size'
	    @summary.sort!() { |a, b| b['size'].to_i() <=> a['size'].to_i() }
	  else
	    @summary.sort!() { |a, b| b[field] <=> a[field] }
	  end
	end
	@summary.reverse!() if /ascending/i =~ @nmzoptions['sort']
      end
      @summary.reverse!() if @nmzoptions['ascending']
      @summary
    end
  end # Summarizer

  class Formatter
    def initialize(nmzindices, nmzoptions = Hash.new(), nmzconfig = Hash.new())
      @nmzindices = nmzindices
      @nmzoptions = nmzoptions
      @nmzconfig = nmzconfig

      if @nmzconfig['template']
	@templatedirname = @nmzconfig['template']
	@templatedirname += File::Separator unless @templatedirname[-1] == File::Separator[0]
      else
	@templatedirname = @nmzindices[0].nmzdirname()
      end
    end

    private
    def read_template(nmztemplatename, querystring = nil)
      nmztemplate = nil
      open(@templatedirname + nmztemplatename) do |file|
	nmztemplate = file.read()
      end
      nmztemplate.gsub!(/<form.*<\/form>/ip, '') unless @nmzoptions['form']
      nmztemplate.gsub!(/\{cgi\}/i, File::basename($0))
      nmztemplate.gsub!(/\$\{(namazu::)?([^\}]+)\}/, '#{\2}')
      if querystring
	querystring = querystring.gsub(/&/n, '&amp;').gsub(/\"/n, '&quot;').gsub(/>/n, '&gt;').gsub(/</n, '&lt;')
	nmztemplate.gsub!(/(<input[^>]+name="query"[^>]*)>/ip,
			  '\1 value="'+ querystring + '">')
	nmztemplate.gsub!(/<\/title>/i, ': &lt;' + querystring + '&gt;\&')
      end
      @nmzoptions.each() do |key, value|
	nmztemplate.gsub!(/<select[^>]+name="#{key}".*?<\/select>/ip) do
	  |selections|
	  selections.gsub!(/\s*selected\s*/ip, ' ')
	  unless selections.gsub!(/(<option[^>]+value="#{value}"[^>]*)>/ip,
				  '\1 selected>')
	    selections.sub!(/<\/select>$/ip,
			    '<option value="' + value.to_s() +
			    '" selected>' + value.to_s() + "\n" + '\&')
	  end
	  selections
	end
      end
      nmztemplate
    end

    private
    def references2s(nmzsummary)
      result = ''
      result.concat("<h2>Results:</h2>\n\n")
      result.concat('<p>References: ')
      result.concat(nmzsummary.hitref())
      result.concat("</p>\n\n")
      result
    end

    private
    def hitnum2s(nmzsummary)
      result = ''
      if (@nmzoptions['count'])
	result.concat('<strong><!-- HIT -->' + nmzsummary.hitnum().to_s() + "<!-- HIT --></strong>\n")
      elsif (!@nmzoptions['list'])
	result.concat('<p><strong> Total <!-- HIT -->' + nmzsummary.hitnum().to_s() + "<!-- HIT --> documents matching your query.</strong></p>\n\n")
      end
      result
    end

    private
    def lists2s(nmzsummary)
      result = ''

      resulttemplate = nil
      if @nmzoptions['list']
	resulttemplate = '<dt>#{uri}'
      else
	if @nmzoptions['result']
	  resulttemplate = read_template('NMZ.result.' + @nmzoptions['result'])
	elsif @nmzoptions['short']
	  resulttemplate = read_template('NMZ.result.short')
	else
	  resulttemplate = read_template('NMZ.result.normal')
	end
      end

      tmpresult = nil
      result.concat("<dl>\n") if @nmzoptions['html']
      counter = nmzsummary.whence() + 1
      for listno in 1 .. nmzsummary.max()
	uri = nmzsummary[counter - 1]['uri']
	if @nmzoptions['decode-uri']
	  uri.gsub!(/\+/n, ' ')
	  uri.gsub!(/%([0-9a-fA-F]{2})/n){ [$1.hex].pack('c') }
	end
	if @nmzoptions['replace'] and @nmzconfig['replace']
	  @nmzconfig['replace'].each() do |value|
	    uri.sub!(*value)
	  end
	end
	title = nmzsummary[counter - 1]['subject']
	score = nmzsummary[counter - 1]['score'].to_i().to_s()
	j = (score.length() - 1) % 3 + 1
	while j < score.length()
	  score[j, 0] = ','
	  j += 4
	end
	author = nmzsummary[counter - 1]['from']
	date = nmzsummary[counter - 1]['date']
	summary = nmzsummary[counter - 1]['summary']
	size = nmzsummary[counter - 1]['size']
	j = (size.length() - 1) % 3 + 1
	while j < size.length()
	  size[j, 0] = ','
	  j += 4
	end
	eval("tmpresult = %Q\x1f" + resulttemplate + "\n\x1f")
	result.concat(tmpresult)
	counter += 1
      end
      result.concat("</dl>\n") if @nmzoptions['html']
      result
    end

    private
    def listsummary2s(nmzsummary)
      result = ''
      startlistno = nmzsummary.whence() + 1
      endlistno = nmzsummary.whence() + nmzsummary.max()
      result.concat('<p><strong>Current List: ' +
		    startlistno.to_s() + ' - ' +
		    endlistno.to_s() + "</strong><br></p>\n")
      result
    end

    private
    def page2s(nmzsummary)
      result = ''
      unless @nmzoptions['max']
	result.concat("<p><strong>Page:</strong> <strong>[1]</strong><br></p>\n")
      else
	result.concat("<p><strong>Page:</strong>\n")

	startindex = nmzsummary.whence()
	step = @nmzoptions['max'].to_i()
	i = startindex
	while i > 0
	  i -= step
	end
	tmpnmzoptions = @nmzoptions.dup()
	querystring = nmzsummary.querystring().gsub(/[^a-zA-Z0-9_\-. ]/n) { sprintf("%%%02X", $&.unpack("C")[0])  } .gsub(/ /n, '+')
	if i < startindex
	  tmpnmzoptions['whence'] = startindex - step
	  result.concat(' <a href="' + File::basename($0) +
			'?query=' + querystring + '&amp;' +
			tmpnmzoptions.to_a().collect() { |opt|
			  opt.join('=')
			} .join('&amp;') + '">[prev]</a>' + "\n")
	end
	counter = 1
	while i < nmzsummary.hitnum()
	  if i == startindex
	    result.concat(' <strong>[' +
			  counter.to_s() + ']</strong>' + "\n")
	  else
	    tmpnmzoptions['whence'] = i
	    result.concat(' <a href="' + File::basename($0) +
			  '?query=' + querystring + '&amp;' +
			  tmpnmzoptions.to_a().collect() { |opt|
			    opt.join('=')
			  } .join('&amp;') + '">[' +
			  counter.to_s() + ']</a>' + "\n")
	  end
	  i += step
	  counter += 1
	end
	if i > startindex + step
	  tmpnmzoptions['whence'] = startindex + step
	  result.concat(' <a href="' + File::basename($0) +
			'?query=' + querystring + '&amp;' +
			tmpnmzoptions.to_a().collect() { |opt|
			  opt.join('=')
			} .join('&amp;') + '">[next]</a>' + "\n")
	end
	result.concat("<br></p>\n")
      end
      result
    end

    private
    def html2text(html)
      text = html.dup()
      text.gsub!(/<!--.*?-->/ip, '')
      text.gsub!(/<.*?>/ip, '')
      text.gsub!(/\n{3,}/ip, "\n\n")
      text
    end

    public
    def format(nmzsummary = nil)
      result = ''

      result.concat(read_template('NMZ.head', nmzsummary ? nmzsummary.querystring(): nil)) if @nmzoptions['html']

      unless nmzsummary
	result.concat(read_template('NMZ.body'))
      else
	if (@nmzoptions['references'] and !@nmzoptions['list'] and !@nmzoptions['count'])
	  result.concat(references2s(nmzsummary))
	end
	result.concat(hitnum2s(nmzsummary))
	unless (@nmzoptions['count'])
	  unless nmzsummary.hitnum() > 0
	    result.concat(read_template('NMZ.tips', nmzsummary.querystring()))
	  else
	    result.concat(lists2s(nmzsummary))
	    result.concat(listsummary2s(nmzsummary)) unless @nmzoptions['list']
	    result.concat(page2s(nmzsummary)) if @nmzoptions['page']
	  end
	end
      end

      result.concat(read_template('NMZ.foot', nmzsummary ? nmzsummary.querystring(): nil)) if @nmzoptions['html']

      result = html2text(result) unless @nmzoptions['html']

      result
    end
  end # Formatter

  class CGIFormatter < Formatter
    def initialize(nmzindices, nmzoptions = Hash.new(), nmzconfig = Hash.new())
      @nmzindices = nmzindices

      (nmzoptions.keys() - ['max', 'whence', 'list', 'short', 'result',
	 'late', 'early', 'sort', 'ascending', 'html', 'references',
	 'page', 'form', 'replace', 'decode-uri']).each() do |key|
	nmzoptions.delete(key)
      end
      nmzoptions.each() do |key, value|
	next unless value.kind_of?(String)
	nmzoptions[key] = value.to_i() if /^[\-\+]*\d+$/ =~ value
	nmzoptions[key] = true if /^(true|on)$/i =~ value
	nmzoptions[key] = false if /^(false|off)$/i =~ value
	nmzoptions[key] = nil if /^nil$/i =~ value
      end
      @nmzoptions = nmzoptions

      @nmzconfig = nmzconfig
      if @nmzconfig['template']
	@templatedirname = @nmzconfig['template']
	@templatedirname += File::Separator unless @templatedirname[-1] == File::Separator[0]
      else
	@templatedirname = @nmzindices[0].nmzdirname()
      end
    end

    public
    def format(nmzsummary = nil)
      result = "Content-Type: text/html\n\n"
      result.concat(super)
      result
    end
  end # CGIFormatter

  class XhtmlBasicCGIFormatter < CGIFormatter
    private
    def references2s(nmzsummary)
      result = ''
      result.concat(nmzsummary.hitref())
      result.concat("<br>\n")
      result
    end

    private
    def hitnum2s(nmzsummary)
      result = ''
      if (@nmzoptions['count'])
	result.concat(nmzsummary.hitnum().to_s() + "\n")
      elsif (!@nmzoptions['list'])
	result.concat('Total <!-- HIT -->' + nmzsummary.hitnum().to_s() + "<!-- HIT --> hits.<br>\n")
      end
      result
    end

    private
    def listsummary2s(nmzsummary)
      result = ''
      result
    end

    private
    def page2s(nmzsummary)
      result = ''
      if @nmzoptions['max']
	startindex = nmzsummary.whence()
	step = @nmzoptions['max'].to_i()
	i = startindex
	while i > 0
	  i -= step
	end
	querystring = nmzsummary.querystring().gsub(/[^a-zA-Z0-9_\-. ]/n) { sprintf("%%%02X", $&.unpack("C")[0])  } .gsub(/ /n, '+')
	if i < startindex
	  result.concat('<a href="' + File::basename($0) +
			'?query=' + querystring +
			'&amp;whence=' + (startindex - step).to_s() +
			'&amp;max=' + step.to_s() +
			(@nmzoptions['sort'] ? '&amp;sort=' + @nmzoptions['sort'] : '') +
			'" accesskey="*">*&lt;=</a>' + "\n")
	end
	startlistno = nmzsummary.whence() + 1
	endlistno = nmzsummary.whence() + nmzsummary.max()
	result.concat('[' + startlistno.to_s() + '-' +
		      endlistno.to_s() + "]\n")
	while i < nmzsummary.hitnum()
	  i += step
	end
	if i > startindex + step
	  result.concat('<a href="' + File::basename($0) +
			'?query=' + querystring +
			'&amp;whence=' + (startindex + step).to_s() +
			'&amp;max=' + step.to_s() +
			(@nmzoptions['sort'] ? '&amp;sort=' + @nmzoptions['sort'] : '') +
			'" accesskey="#">=&gt;#</a>' + "\n")
	end
	result.concat("<br>\n")
      end
      result
    end

    public
    def format(nmzsummary = nil)
      result = "Content-Type: text/html\n\n"
      result.concat(read_template('NMZ.head', nmzsummary ? nmzsummary.querystring(): nil)) if @nmzoptions['html']
      unless nmzsummary
	result.concat(read_template('NMZ.body'))
      else
	result.concat(hitnum2s(nmzsummary))
	unless (@nmzoptions['count'])
	  unless nmzsummary.hitnum() > 0
	    result.concat(read_template('NMZ.tips', nmzsummary.querystring()))
	  else
	    result.concat(page2s(nmzsummary)) if @nmzoptions['page']
	    result.concat(lists2s(nmzsummary))
	  end
	end
	if (@nmzoptions['references'] and !@nmzoptions['list'] and !@nmzoptions['count'])
	  result.concat(references2s(nmzsummary))
	end
      end
      result.concat(read_template('NMZ.foot', nmzsummary ? nmzsummary.querystring() : nil)) if @nmzoptions['html']
      result
    end
  end # XhtmlBasicCGIFormatter
end # Namazu
